﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Clients.Common;
using HeuristicLab.Collections;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.PluginInfrastructure;

namespace HeuristicLab.Clients.OKB.Administration {
  [Item("AdministrationClient", "OKB administration client.")]
  public sealed class AdministrationClient : IContent {
    private static AdministrationClient instance;
    public static AdministrationClient Instance {
      get {
        if (instance == null) instance = new AdministrationClient();
        return instance;
      }
    }

    #region Properties
    private ItemCollection<Platform> platforms;
    public ItemCollection<Platform> Platforms {
      get { return platforms; }
    }
    private ItemCollection<AlgorithmClass> algorithmClasses;
    public ItemCollection<AlgorithmClass> AlgorithmClasses {
      get { return algorithmClasses; }
    }
    private ItemCollection<Algorithm> algorithms;
    public ItemCollection<Algorithm> Algorithms {
      get { return algorithms; }
    }
    private ItemCollection<ProblemClass> problemClasses;
    public ItemCollection<ProblemClass> ProblemClasses {
      get { return problemClasses; }
    }
    private ItemCollection<Problem> problems;
    public ItemCollection<Problem> Problems {
      get { return problems; }
    }
    #endregion

    private AdministrationClient() {
      platforms = new ItemCollection<Platform>();
      platforms.ItemsRemoved += new CollectionItemsChangedEventHandler<Platform>(platforms_ItemsRemoved);
      algorithmClasses = new ItemCollection<AlgorithmClass>();
      algorithmClasses.ItemsRemoved += new CollectionItemsChangedEventHandler<AlgorithmClass>(algorithmClasses_ItemsRemoved);
      algorithms = new ItemCollection<Algorithm>();
      algorithms.ItemsRemoved += new CollectionItemsChangedEventHandler<Algorithm>(algorithms_ItemsRemoved);
      problemClasses = new ItemCollection<ProblemClass>();
      problemClasses.ItemsRemoved += new CollectionItemsChangedEventHandler<ProblemClass>(problemClasses_ItemsRemoved);
      problems = new ItemCollection<Problem>();
      problems.ItemsRemoved += new CollectionItemsChangedEventHandler<Problem>(problems_ItemsRemoved);
    }

    #region Refresh
    public void Refresh() {
      OnRefreshing();

      platforms.Clear();
      algorithmClasses.Clear();
      algorithms.Clear();
      problemClasses.Clear();
      problems.Clear();

      var call = new Func<Exception>(delegate() {
        try {
          platforms.AddRange(CallAdministrationService<List<Platform>>(s => s.GetPlatforms()).OrderBy(x => x.Name));
          algorithmClasses.AddRange(CallAdministrationService<List<AlgorithmClass>>(s => s.GetAlgorithmClasses()).OrderBy(x => x.Name));
          algorithms.AddRange(CallAdministrationService<List<Algorithm>>(s => s.GetAlgorithms()).OrderBy(x => x.Name));
          problemClasses.AddRange(CallAdministrationService<List<ProblemClass>>(s => s.GetProblemClasses()).OrderBy(x => x.Name));
          problems.AddRange(CallAdministrationService<List<Problem>>(s => s.GetProblems()).OrderBy(x => x.Name));
          return null;
        }
        catch (Exception ex) {
          return ex;
        }
      });
      call.BeginInvoke(delegate(IAsyncResult result) {
        Exception ex = call.EndInvoke(result);
        if (ex != null) ErrorHandling.ShowErrorDialog("Refresh failed.", ex);
        OnRefreshed();
      }, null);
    }
    #endregion

    #region Store
    public bool Store(IOKBItem item) {
      try {
        if (item.Id == 0) {
          if (item is Platform)
            item.Id = CallAdministrationService<long>(s => s.AddPlatform((Platform)item));
          else if (item is AlgorithmClass)
            item.Id = CallAdministrationService<long>(s => s.AddAlgorithmClass((AlgorithmClass)item));
          else if (item is Algorithm)
            item.Id = CallAdministrationService<long>(s => s.AddAlgorithm((Algorithm)item));
          else if (item is ProblemClass)
            item.Id = CallAdministrationService<long>(s => s.AddProblemClass((ProblemClass)item));
          else if (item is Problem)
            item.Id = CallAdministrationService<long>(s => s.AddProblem((Problem)item));
        } else {
          if (item is Platform)
            CallAdministrationService(s => s.UpdatePlatform((Platform)item));
          else if (item is AlgorithmClass)
            CallAdministrationService(s => s.UpdateAlgorithmClass((AlgorithmClass)item));
          else if (item is Algorithm)
            CallAdministrationService(s => s.UpdateAlgorithm((Algorithm)item));
          else if (item is ProblemClass)
            CallAdministrationService(s => s.UpdateProblemClass((ProblemClass)item));
          else if (item is Problem)
            CallAdministrationService(s => s.UpdateProblem((Problem)item));
        }
        return true;
      }
      catch (Exception ex) {
        ErrorHandling.ShowErrorDialog("Store failed.", ex);
        return false;
      }
    }
    #endregion

    #region Algorithm Methods
    public List<Guid> GetAlgorithmUsers(long algorithmId) {
      try {
        return CallAdministrationService<List<Guid>>(s => s.GetAlgorithmUsers(algorithmId));
      }
      catch (Exception ex) {
        ErrorHandling.ShowErrorDialog("Refresh authorized algorithm users failed.", ex);
        return null;
      }
    }
    public bool UpdateAlgorithmUsers(long algorithmId, List<Guid> users) {
      try {
        CallAdministrationService(s => s.UpdateAlgorithmUsers(algorithmId, users));
        return true;
      }
      catch (Exception ex) {
        ErrorHandling.ShowErrorDialog("Update authorized algorithm users failed.", ex);
        return false;
      }
    }
    public byte[] GetAlgorithmData(long algorithmId) {
      try {
        return CallAdministrationService<byte[]>(s => s.GetAlgorithmData(algorithmId));
      }
      catch (Exception ex) {
        ErrorHandling.ShowErrorDialog("Refresh algorithm data failed.", ex);
        return null;
      }
    }
    public bool UpdateAlgorithmData(long algorithmId, byte[] algorithmData) {
      try {
        CallAdministrationService(s => s.UpdateAlgorithmData(algorithmId, algorithmData));
        return true;
      }
      catch (Exception ex) {
        ErrorHandling.ShowErrorDialog("Update algorithm data failed.", ex);
        return false;
      }
    }
    #endregion

    #region Problem Methods
    public List<Guid> GetProblemUsers(long problemId) {
      try {
        return CallAdministrationService<List<Guid>>(s => s.GetProblemUsers(problemId));
      }
      catch (Exception ex) {
        ErrorHandling.ShowErrorDialog("Refresh authorized problem users failed.", ex);
        return null;
      }
    }
    public bool UpdateProblemUsers(long problemId, List<Guid> users) {
      try {
        CallAdministrationService(s => s.UpdateProblemUsers(problemId, users));
        return true;
      }
      catch (Exception ex) {
        ErrorHandling.ShowErrorDialog("Update authorized problem users failed.", ex);
        return false;
      }
    }
    public byte[] GetProblemData(long problemId) {
      try {
        return CallAdministrationService<byte[]>(s => s.GetProblemData(problemId));
      }
      catch (Exception ex) {
        ErrorHandling.ShowErrorDialog("Refresh problem data failed.", ex);
        return null;
      }
    }
    public bool UpdateProblemData(long problemId, byte[] problemData) {
      try {
        CallAdministrationService(s => s.UpdateProblemData(problemId, problemData));
        return true;
      }
      catch (Exception ex) {
        ErrorHandling.ShowErrorDialog("Update problem data failed.", ex);
        return false;
      }
    }
    #endregion

    #region Events
    public event EventHandler Refreshing;
    private void OnRefreshing() {
      EventHandler handler = Refreshing;
      if (handler != null) handler(this, EventArgs.Empty);
    }
    public event EventHandler Refreshed;
    private void OnRefreshed() {
      EventHandler handler = Refreshed;
      if (handler != null) handler(this, EventArgs.Empty);
    }

    private void platforms_ItemsRemoved(object sender, CollectionItemsChangedEventArgs<Platform> e) {
      try {
        foreach (Platform p in e.Items)
          CallAdministrationService(s => s.DeletePlatform(p.Id));
      }
      catch (Exception ex) {
        ErrorHandling.ShowErrorDialog("Delete failed.", ex);
      }
    }
    private void algorithmClasses_ItemsRemoved(object sender, CollectionItemsChangedEventArgs<AlgorithmClass> e) {
      try {
        foreach (AlgorithmClass a in e.Items)
          CallAdministrationService(s => s.DeleteAlgorithmClass(a.Id));
      }
      catch (Exception ex) {
        ErrorHandling.ShowErrorDialog("Delete failed.", ex);
      }
    }
    private void algorithms_ItemsRemoved(object sender, CollectionItemsChangedEventArgs<Algorithm> e) {
      try {
        foreach (Algorithm a in e.Items)
          CallAdministrationService(s => s.DeleteAlgorithm(a.Id));
      }
      catch (Exception ex) {
        ErrorHandling.ShowErrorDialog("Delete failed.", ex);
      }
    }
    private void problemClasses_ItemsRemoved(object sender, CollectionItemsChangedEventArgs<ProblemClass> e) {
      try {
        foreach (ProblemClass p in e.Items)
          CallAdministrationService(s => s.DeleteProblemClass(p.Id));
      }
      catch (Exception ex) {
        ErrorHandling.ShowErrorDialog("Delete failed.", ex);
      }
    }
    private void problems_ItemsRemoved(object sender, CollectionItemsChangedEventArgs<Problem> e) {
      try {
        foreach (Problem p in e.Items)
          CallAdministrationService(s => s.DeleteProblem(p.Id));
      }
      catch (Exception ex) {
        ErrorHandling.ShowErrorDialog("Delete failed.", ex);
      }
    }
    #endregion

    #region Helpers
    private void CallAdministrationService(Action<IAdministrationService> call) {
      AdministrationServiceClient client = ClientFactory.CreateClient<AdministrationServiceClient, IAdministrationService>();
      try {
        call(client);
      }
      finally {
        try {
          client.Close();
        }
        catch (Exception) {
          client.Abort();
        }
      }
    }
    private T CallAdministrationService<T>(Func<IAdministrationService, T> call) {
      AdministrationServiceClient client = ClientFactory.CreateClient<AdministrationServiceClient, IAdministrationService>();
      try {
        return call(client);
      }
      finally {
        try {
          client.Close();
        }
        catch (Exception) {
          client.Abort();
        }
      }
    }
    #endregion
  }
}
